package App::githook_perltidy;
use strict;

our $VERSION = '1.0.1';

1;

__END__

=head1 NAME

App::githook_perltidy - legacy package for dependencies

=head1 VERSION

1.0.1 (2022-10-14)

=head1 DESCRIPTION

This package only exists for backwards compatibility: so that authors
that recommend or require App::githook_perltidy still get the new
App::githook::perltidy based version of githook-perltidy.

=head1 SEE ALSO

L<githook-perltidy>(1)

=head1 AUTHOR

Mark Lawrence E<lt>nomad@null.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2011-2022 Mark Lawrence <nomad@null.net>

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

